package com.taauav.admin.controller;

import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.SysAuthGroup;
import com.taauav.admin.service.ISysAuthGroupService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户组表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@RestController
@RequestMapping("/authgroup")
public class SysAuthGroupController {
    @Autowired
    private ISysAuthGroupService iSysAuthGroupService;
    @Autowired
    private Response response;

    private static final String controllerName = "authgroup";

    /**
     * 角色列表
     * @param map
     * @return
     */
    @GetMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) Map<String,String> map) {
        return iSysAuthGroupService.getList(map);
    }

    /**
     * 修改角色
     * @param authGroup
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody(required = false) @Valid SysAuthGroup authGroup, BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iSysAuthGroupService.edit(authGroup);
    }

    /**
     * 获取角色数据
     * @param id
     * @return
     */
    @GetMapping("/getAuthGroup")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getAuthGroup(Integer id) {
        if (id <= 0) {
            response.failure("数据不存在");
        }
        Map<String,Object> map = new HashMap<>();
        SysAuthGroup info = iSysAuthGroupService.getInfoById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        map.put("id", info.getId());
        map.put("name",info.getName());
        map.put("pid",info.getPid());
        map.put("has_child",info.getHasChild());
        map.put("remark",info.getRemark());
        map.put("sort",info.getSort());
        List<Map<String,String>> pList ;
        if (info.getPid() > 0) {
            pList = iSysAuthGroupService.getListByPid(0);
        } else {
            pList = null;
        }
        map.put("p_list",pList);
        return response.success(map);
    }

    /**
     * 添加角色
     * @param authGroup
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid SysAuthGroup authGroup,BindingResult bindingResult) {
        if(bindingResult.hasErrors()){
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iSysAuthGroupService.add(authGroup);
    }

    /**
     * 删除角色
     * @param authGroup
     * @return
     */
    @PostMapping("/del")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response del(@RequestBody SysAuthGroup authGroup) {
        return iSysAuthGroupService.del(authGroup);
    }

    /**
     * 设置权限
     * @param map
     * @return
     */
    @PostMapping("/setRules")
    @RequiresPermissions("authgroup:setRules")
    public Response setRules(@RequestBody Map<String,String> map) {
        return iSysAuthGroupService.setRules(map);
    }

    /**
     * 后台角色设置状态
     * @param map
     * @return
     */
    @PostMapping("/editStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response editStatus(@RequestBody Map<String,String> map) {
        return iSysAuthGroupService.editStatus(map);
    }
}
