package com.taauav.admin.controller;


import com.taauav.admin.entity.SysCity;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.math.BigInteger;
import java.util.List;

/**
 * <p>
 * 城市表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@RestController
@RequestMapping("/syscity")
public class SysCityController {

    @Autowired
    private ISysCityService cityService;

    @Autowired
    private Response response;

    private static final String controllerName = "syscity";

    /**
     * 获取机构列表
     *
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index() {
        return cityService.getList();
    }

    /**
     * 添加机构
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid SysCity entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return cityService.add(entity);
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(BigInteger id) {
        return cityService.info(id);
    }

    /**
     * 编辑机构
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid SysCity entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return cityService.edit(entity);
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(BigInteger id) {
        return cityService.drop(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody SysCity entity) {
        return cityService.setStatus(entity);
    }

    /**
     * 获取城市子级列表
     *
     * @param pid 上级ID
     * @return
     */
    @GetMapping("/getChildCityList")
    public Response getChildCityList(BigInteger pid) {
        return response.success(cityService.getChildCityList(pid));
    }

    /**
     * 获取区划列表
     *
     * @return
     */
    @PostMapping("/getCityList")
    public Response getCityList() {
        List<SysCity> cityList = cityService.getCityList(BigInteger.valueOf(Long.valueOf("320117")));
        return response.success(cityList);
    }

}
