package com.taauav.admin.controller;


import com.taauav.admin.entity.TauvMountEquipment;
import com.taauav.admin.query.MountEquipmentQuery;
import com.taauav.admin.service.ITauvMountEquipmentService;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Map;

/**
 * <p>
 * 挂载设备表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@RestController
@RequestMapping("/mountequipment")
public class TauvMountEquipmentController extends BaseController {

    @Autowired
    private ITauvMountEquipmentService tauvMountEquipmentService;

    @Autowired
    private Response response;

    private static final String controllerName = "mountequipment";

    /**
     * 获取挂载设备列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) MountEquipmentQuery query) {
        return tauvMountEquipmentService.getList(query);
    }

    /**
     * 添加挂载设备
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid TauvMountEquipment entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return tauvMountEquipmentService.add(entity);
    }

    /**
     * 获取挂载设备详情
     *
     * @param id 挂载设备ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return tauvMountEquipmentService.info(id);
    }

    /**
     * 编辑挂载设备
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid TauvMountEquipment entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return tauvMountEquipmentService.edit(entity);
    }

    /**
     * 删除挂载设备
     *
     * @param id 设备ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return tauvMountEquipmentService.drop(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(TauvMountEquipment entity) {
        return tauvMountEquipmentService.setStatus(entity);
    }

    /**
     * 获取设备参数
     *
     * @return
     */
    @PostMapping("/getParamsList")
    public Response getParamsList() {
        return tauvMountEquipmentService.getParamsList();
    }

    /**
     * 获取挂载设备列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/getMountEquipmentList")
    public Response getMountEquipmentList(@RequestBody MountEquipmentQuery query) {
        return tauvMountEquipmentService.getMountEquipmentList(query);
    }

}
