package com.taauav.admin.controller;

import com.taauav.admin.entity.TauvWaterAlarm;
import com.taauav.admin.query.TauvWaterAlarmQuery;
import com.taauav.admin.service.ITauvWaterAlarmService;
import com.taauav.admin.vo.TauvWaterAlarmVo;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;

/**
 * 水质超标信息 控制器
 *
 * @author daixiantong
 * @date 2020-05-14
 */
@RestController
@RequestMapping("/admin/waterAlarm")
public class TauvWaterAlarmController extends BaseController{
    private static final String controllerName = "admin:waterAlarm";
    @Autowired
    private ITauvWaterAlarmService waterAlarmService;
    @Resource
    private Response response;

    /**
     * 水质超标报警列表
     * @param waterAlarmQuery
     * @return
     */
    @GetMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(TauvWaterAlarmQuery waterAlarmQuery) {
        return waterAlarmService.selectPageList(waterAlarmQuery);
    }

    /**
     * 报警详情
     * @param id
     * @return
     */
    @GetMapping("/detail")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.VIEW_PERMISSION)
    public Response getDetail(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("id不能为空");
        }
        TauvWaterAlarmVo waterAlarmVo = waterAlarmService.getDetail(id);
        return response.success(waterAlarmVo);
    }

    /**
     * 修改报警状态
     * @param waterAlarm
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response changeStatus(TauvWaterAlarm waterAlarm) {
        Boolean res = waterAlarmService.editData(waterAlarm);
        if (!res) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }
}
