package com.taauav.admin.controller;

import com.taauav.common.bean.Response;
import com.taauav.common.util.ImageUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;

/**
 * <p>文件上传</p>
 *
 * @author : dyg
 * @date : 2019-11-23 13:21
 **/
@RestController
@Slf4j
@RequestMapping("/upload")
public class UploadController {
    @Value("${file.uploadFolder}")
    private String uploadFolder;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Autowired
    private Response response;

    /**
     * 文件上传
     *
     * @param file
     * @return
     */
    @RequestMapping("upload")
    public Response upload(@RequestParam("file") MultipartFile file) {
        if (file.isEmpty()) {
            return response.failure("上传失败，请选择文件");
        }
        Map<String, String> preUrl = getUploadFile(file);
        if (preUrl.get("success").equals("0")) {
            return response.failure("上传失败!" + preUrl.get("msg"));
        }
        return response.success("上传成功", preUrl);
    }

    /**
     * 文件批量上传
     *
     * @param fileList
     * @return
     */
    @PostMapping("multiUpload")
    public Response mulityUpload(@RequestParam("file") List<MultipartFile> fileList) {
        if (fileList.isEmpty()) {
            return response.failure("上传失败，请选择文件");
        }
        List<Map<String, String>> list = new ArrayList<>();
        for (MultipartFile file : fileList) {
            Map<String, String> preFile = getUploadFile(file);
            list.add(preFile);
        }
        return response.success("上传成功", list);
    }

    /**
     * 文件上传处理
     *
     * @param file
     * @return
     */
    private Map<String, String> getUploadFile(MultipartFile file) {
        Map<String, String> dt = new HashMap<String, String>();
        String fileName = file.getOriginalFilename();
        String extension = fileName.substring(fileName.lastIndexOf(".") + 1);
        String newFileName = UUID.randomUUID().toString().replace("-", "").substring(0, 10) + "." + extension;
        String uploadPath = mkfilePath();
        String fileUrl = uploadPath + newFileName;
        File dest = new File(fileUrl);

        dt.put("fileName", fileName);
        try {
            file.transferTo(dest);
            String preUrl = "";
            if (ImageUtil.isImage(extension)) {
                //生成缩略图
                List<String> thumImgs = ImageUtil.generateThumbnail2Directory(uploadPath, fileUrl);
                preUrl = thumImgs.get(0).replace(uploadFolder, uploadUrl);
            } else {
                preUrl = fileUrl.replace(uploadFolder, uploadUrl);
            }

            dt.put("url", preUrl);
            dt.put("success", "1");
            dt.put("msg", "ok");
        } catch (IOException e) {
            e.printStackTrace();
            log.error("文件上传异常" + e.getMessage());
            dt.put("url", "");
            dt.put("success", "0");
            dt.put("msg", e.getMessage());
        }
        return dt;
    }

    /**
     * 上传文件路径创建
     *
     * @return
     */
    private String mkfilePath() {
        Calendar date = Calendar.getInstance();
        String year = String.valueOf(date.get(Calendar.YEAR));
        String month = String.valueOf(date.get(Calendar.MONTH) + 1);
        String day = String.valueOf(date.get(Calendar.DATE));

        String uploadPath = uploadFolder + "temp/" + year + "/" + month + "/" + day + "/";
        File pathFile = new File(uploadPath);
        if (!pathFile.isDirectory()) {
            pathFile.mkdirs();
        }

        return uploadPath;
    }
}
