package com.taauav.admin.entity;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;

/**
 * <p>
 * 设备管理表
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class TauvEquipment extends Entity {

    private static final long serialVersionUID = 1L;
    /**
     * 设备编号
     */
    private String equipmentSn;

    /**
     * 设备名称
     */
    @NotEmpty(message = "设备名称不能为空")
    @Length(min = 2, max = 20, message = "设备名称长度为2-20个字")
    private String name;

    /**
     * 品牌
     */
    @NotEmpty(message = "品牌不能为空")
    @Length(max = 100, message = "品牌长度不能超过100个字")
    private String brand;

    /**
     * 型号
     */
    @Length(max = 100, message = "型号长度不能超过100个字")
    private String model;

    /**
     * 产品序号
     */
    @NotEmpty(message = "产品序号不能空")
    @Length(min = 2, max = 150, message = "产品序号长度为2-150个字")
    private String productSn;

    /**
     * 飞控序号
     */
    @Length(max = 150, message = "飞控序号长度不能超过150个字")
    private String flightSn;

    /**
     * 类型：1多旋翼 2固定翼
     */
    private Integer type;

    /**
     * 任务类型：1可见光 2多光谱
     */
    private String taskType;

    /**
     * 注册时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date registerTime;

    public Map<Integer, String> getStatusList() {
        Map<Integer, String> map = new HashMap<>();
        map.put(1, "正常");
        map.put(2, "停用");
        return map;
    }

    public Map<Integer, String> getTypeList() {
        Map<Integer, String> map = new HashMap<>();
        map.put(1, "多旋翼");
        map.put(2, "固定翼");
        map.put(3, "垂直起降");
        return map;
    }

    /**
     * 任务类型
     *
     * @return
     */
    public Map<Integer, String> getTaskTypeList() {
        Map<Integer, String> map = new HashMap<>();
        map.put(1, "可见光");
        map.put(2, "多光谱");
        return map;
    }
}
