package com.taauav.admin.entity;

import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.TableField;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.*;

/**
 * <p>
 * 巡检任务表
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class TauvInspect extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 巡检任务单号
     */
    private String inspectNo;

    /**
     * 类型：1可见光巡检 2多光谱巡检
     */
    @NotNull(message = "巡检类型不能为空")
    private Integer type;

    /**
     * 巡检时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date inspectTime;

    /**
     * 实际执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date executionTime;

    /**
     * 巡检总长(单位：KM)
     */
    @NotNull(message = "巡检总长度不能为空")
    private BigDecimal inspectLength;

    /**
     * 备注
     */
    private String note;

    /**
     * 附件地址
     */
    private String attachment;

    /**
     * 河道任务
     */
    @TableField(exist = false)
    private List<TauvInspectDriver> inspectDriverList;

    /**
     * 类型数组
     *
     * @return
     */
    public Map<Integer, String> getTypeList() {
        Map<Integer, String> map = new HashMap<>(2);
        map.put(1, "可见光巡检");
        map.put(2, "多光谱巡检");
        return map;
    }

    /**
     * 状态数组
     *
     * @return
     */
    public Map<Integer, String> getStatusList() {
        Map<Integer, String> map = new HashMap<>(5);
        map.put(2, "待执行");
        map.put(3, "执行中");
        map.put(4, "已完成");
        map.put(5, "复核中");
        map.put(6, "复核完成");
        return map;
    }
}
