package com.taauav.admin.entity;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;

/**
 * <p>
 * 问题选项管理表
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class TauvQuestionOptions extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 分类：1水面 2岸线 3排口 4水质
     */
    @NotNull(message = "请选项分类")
    private Integer category;

    /**
     * 类型：1日常监测 2重点核查
     */
    @NotNull(message = "请选择类型")
    private Integer type;

    /**
     * 问题选项
     */
    @NotEmpty(message = "问题描述不能为空")
    @Length(min = 2, max = 200, message = "问题描述长度为2-200字")
    private String content;

    /**
     * 备注
     */
    @Length(max = 200, message = "备注长度不能超过200字")
    private String note;

    /**
     * 基础分值
     */
    @NotNull(message = "基础分值不能为空")
    @Max(value = 99, message = "最大基础分值不能超过99")
    @Min(value = 0, message = "最小基础分值不能小于0")
    private Integer baseScore;

    /**
     * 国考基础分值
     */
    @NotNull(message = "国考基础分值不能为空")
    @Max(value = 99, message = "最大国考基础分值不能超过99")
    @Min(value = 0, message = "最小国考基础分值不能小于0")
    private Integer examBaseScore;

    /**
     * 没增加问题数量
     */
    private Integer increaseNum;

    /**
     * 扣分
     */
    private Integer deduction;

    /**
     * 排序
     */
    private Integer sort;

    public Map<Integer, String> getStatusList() {
        Map<Integer, String> map = new HashMap<>(2);
        map.put(1, "正常");
        map.put(2, "禁用");
        return map;
    }

    public Map<Integer, String> getCategoryList() {
        Map<Integer, String> map = new HashMap<>(4);
        map.put(1, "水面");
        map.put(2, "岸线");
        map.put(3, "排口");
        map.put(4, "水质");
        return map;
    }

    public Map<Integer, String> getTypeList() {
        Map<Integer, String> map = new HashMap<>(2);
        map.put(1, "日常监测");
        map.put(2, "重点检查");
        return map;
    }

}
