package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysConfig;
import com.taauav.admin.mapper.SysConfigMapper;
import com.taauav.admin.service.ISysConfigService;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.validation.ConstraintViolation;
import javax.validation.Validator;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.Set;

/**
 * <p>
 * 系统配置表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-12
 */
@Service
public class SysConfigServiceImpl extends BaseServiceImpl<SysConfigMapper, SysConfig> implements ISysConfigService {
    @Autowired
    private Response response;
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Autowired
    private SysConfigMapper configMapper;
    @Resource
    private Validator validator;

    /**
     * 获取分页列表数据
     *
     * @param map
     * @return
     */
    @Override
    public Response getPageList(Map<String, String> map) {
        String group = "group";
        if (map == null || !map.containsKey(group)) {
            return response.failure("请指定配置分组");
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("`mark`", 1);
        wrapper.eq("`group`", Integer.valueOf(map.get(group)));
        //关键词筛选
        String keywords = "keywords";
        if (map.containsKey(keywords) && !"".equals(map.get(keywords))) {
            if (FunctionUtils.isNumeric(map.get(keywords))) {
                wrapper.eq("`id`", Integer.valueOf(map.get(keywords)));
            } else {
                wrapper.like("`name`", map.get(keywords));
            }
        }
        //配置类型筛选
        String type = "type";
        if (map.containsKey(type) && !"".equals(map.get(type))) {
            wrapper.eq("`type`", map.get(type));
        }
        //输入类型筛选
        String inputType = "inputType";
        if (map.containsKey(inputType) && !"".equals(map.get(inputType))) {
            wrapper.eq("`input_type`", map.get(inputType));
        }
        //状态筛选
        String status = "status";
        if (map.containsKey(status) && !"".equals(map.get(status))) {
            wrapper.eq("`status`", Integer.valueOf(map.get(status)));
        }
        //添加人筛选
        String createUser = "createUser";
        if (map.containsKey(createUser) && !"".equals(map.get(createUser))) {
            wrapper.eq("`create_user`", Integer.valueOf(map.get(createUser)));
        }
        //添加时间筛选
        String startTime = "startTime";
        String endTime = "endTime";
        if (map.containsKey(startTime) && map.containsKey(endTime) && FunctionUtils.isNumeric(map.get(startTime)) && FunctionUtils.isNumeric(map.get(endTime))) {
            wrapper.between("`create_time`", Integer.valueOf(map.get(startTime)), Integer.valueOf(map.get(endTime)));
        }
        //每页条数
        String pgSize = "pageSize";
        if (map.containsKey(pgSize) && Integer.valueOf(map.get(pgSize)) > 0) {
            pageSize = Integer.valueOf(map.get(pgSize));
        }
        Integer pageNo = map.containsKey("page") && Integer.valueOf(map.get("page")) > 0 ? Integer.valueOf(map.get("page")) : 1;
        IPage<SysConfig> page = new Page<>(pageNo, pageSize);
        IPage<Map<String, Object>> data = configMapper.selectMapsPage(page, wrapper);
        SysConfig sysConfig = new SysConfig();
        Map<String, String> inputTypeList = sysConfig.getInputTypeList();
        Map<Integer, String> statusList = sysConfig.getStatusList();
        if (!StringUtils.isEmpty(data.getRecords())) {
            for (Map<String, Object> mp : data.getRecords()) {
                mp.put("statusText", statusList.get(mp.get("status")));
                mp.put("inputTypeText", inputTypeList.get(mp.get("input_type")));
            }
        }
        return response.success("操作成功", data);
    }

    /**
     * 根据id获取配置数据
     *
     * @param id
     * @return
     */
    @Override
    public SysConfig getConfigById(Integer id) {
        SysConfig configInfo = configMapper.selectById(id);
        return configInfo;
    }

    /**
     * 添加、编辑配置数据
     *
     * @param map
     * @return
     */
    @Override
    public Response editData(SysConfig map) {
        //必填字段及格式校验
        Set<ConstraintViolation<SysConfig>> violations = validator.validate(map);
        for (ConstraintViolation<SysConfig> violation : violations) {
            if (!"".equals(violation.getMessage())) {
                return response.failure(violation.getMessage());
            }
        }
        if (map.getId() != null && map.getId() > 0) {
            map.setUpdateTime(DateUtil.now());
            map.setUpdateUser(ShiroUtils.getAdminId());
            updateById(map);
        } else {
            map.setCreateTime(DateUtil.now());
            map.setCreateUser(ShiroUtils.getAdminId());
            save(map);
        }
        return response.success("操作成功");
    }

    /**
     * 设置配置状态
     *
     * @param map
     * @return
     */
    @Override
    public Response setStatus(Map<String, Integer> map) {
        Integer id = map.getOrDefault("id", 0);
        if (id == 0) {
            return response.failure("配置id不能为空");
        }
        Integer status = map.getOrDefault("status", 0);
        if (status == 0) {
            return response.failure("状态值不能为空");
        }
        SysConfig config = configMapper.selectById(id);
        if (config == null) {
            return response.failure("对应配置数据不存在");
        }
        if (status == 1) {
            config.setStatus(Integer.valueOf("2"));
        } else {
            config.setStatus(Integer.valueOf("1"));
        }
        updateById(config);
        return response.success("操作成功");
    }
}
