package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.exceptions.ApiException;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.Joiner;
import com.taauav.admin.entity.*;
import com.taauav.admin.mapper.*;
import com.taauav.admin.query.TauvWaterDataQuery;
import com.taauav.admin.service.ITauvWaterAlarmInfoService;
import com.taauav.admin.service.ITauvWaterAlarmService;
import com.taauav.admin.service.ITauvWaterDataService;
import com.taauav.admin.vo.TauvWaterDataVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.math.BigDecimal;
import java.util.*;

/**
 * 水质数据 服务类
 *
 * @author daixiantong
 * @date 2020-05-14
 */
@Service
public class TauvWaterDataServiceImpl extends BaseServiceImpl<TauvWaterDataMapper, TauvWaterData> implements ITauvWaterDataService {
    @Resource
    private Response response;
    @Autowired
    private SysCityMapper cityMapper;
    @Autowired
    private TauvInspectDriverMapper inspectDriverMapper;
    @Autowired
    private TauvDriverMapper driverMapper;
    @Autowired
    private TauvWaterStandardMapper waterStandardMapper;
    @Autowired
    private TauvWaterAlarmMapper waterAlarmMapper;
    @Autowired
    private ITauvWaterAlarmInfoService waterAlarmInfoService;
    @Autowired
    private ITauvWaterAlarmService waterAlarmService;
    @Value("${file.uploadFolder}")
    private String uploadFolder;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;

    /**
     * 自定义分页
     *
     * @param waterDataQuery
     * @return
     */
    @Override
    public Response selectPageList(TauvWaterDataQuery waterDataQuery) {
        IPage<TauvWaterData> page = new Page<>(waterDataQuery.getPage(), waterDataQuery.getPageSize());
        IPage<TauvWaterDataVo> list = baseMapper.selectPageList(page, waterDataQuery);
        if (list.getRecords().size() > 0) {
            for (TauvWaterDataVo data : list.getRecords()) {
                SysCity city = cityMapper.selectById(data.getDriverArea());
                data.setAreaName(city == null ? "" : city.getName());
            }
        }
        return response.success(list);
    }

    /**
     * 导入数据
     *
     * @param file
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Response importData(MultipartFile file) {
        List<List<Object>> objectList = null;
        try {
            List<String> headList = ExcelUtil.getHeadList(file, 0, 0);
            String[] titleList = new String[]{"*任务单号", "*区属", "*河湖名称", "化学需氧量", "氨氮", "总磷", "总氮", "溶解氧", "浊度"};
            for (int i = 0; i < titleList.length; i++) {
                if (!titleList[i].equals(headList.get(i))) {
                    throw new ApiException("请使用正确的导入模板");
                }
            }
            objectList = ExcelUtil.importExcel(file, 0, 0);
        } catch (Exception e) {
            e.printStackTrace();
            return response.failure(e.getMessage());
        }
        if (objectList.size() == 0) {
            throw new ApiException("导入数据不能为空");
        }
        Integer m = 2;
        for (List<Object> data : objectList) {
            String inspectNo = data.get(0).toString().trim();
            String areaName = data.get(1).toString().trim();
            String driverName = data.get(2).toString().trim();
            if (StringUtils.isEmpty(inspectNo)) {
                throw new ApiException("第" + m + "行任务单号不能为空");
            }
            if (StringUtils.isEmpty(areaName)) {
                throw new ApiException("第" + m + "行区属不能为空");
            }
            if (StringUtils.isEmpty(driverName)) {
                throw new ApiException("第" + m + "行河湖名称不能为空");
            }

            // 任务校验
            QueryWrapper inspectWrapper = new QueryWrapper();
            inspectWrapper.eq("mark", 1);
            inspectWrapper.eq("inspect_no", inspectNo);
            inspectWrapper.last("limit 1");
            TauvInspectDriver inspectDriver = inspectDriverMapper.selectOne(inspectWrapper);
            if (inspectDriver == null) {
                throw new ApiException("第" + m + "行任务单号错误");
            }
            if (inspectDriver.getStatus() != 4) {
                throw new ApiException("第" + m + "行任务未完成");
            }
            // 区属校验
            QueryWrapper cityWrapper = new QueryWrapper();
            cityWrapper.eq("mark", 1);
            cityWrapper.eq("name", areaName);
            cityWrapper.last("limit 1");
            SysCity city = cityMapper.selectOne(cityWrapper);
            if (city == null) {
                throw new ApiException("第" + m + "行区属错误");
            }
            // 河湖校验
            QueryWrapper driverWrapper = new QueryWrapper();
            driverWrapper.eq("mark", 1);
            driverWrapper.eq("name", driverName);
            driverWrapper.eq("driver_area", city.getId());
            driverWrapper.last("limit 1");
            TauvDriver driver = driverMapper.selectOne(driverWrapper);
            if (driver == null) {
                throw new ApiException("第" + m + "行河湖名称错误");
            }
            // 关系校验
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("inspect_no", inspectNo);
            wrapper.eq("driver_area", city.getId());
            wrapper.eq("driver_id", driver.getId());
            wrapper.eq("driver_name", driverName);
            wrapper.last("limit 1");
            TauvInspectDriver tauvInspectDriver = inspectDriverMapper.selectOne(wrapper);
            if (tauvInspectDriver == null) {
                throw new ApiException("第" + m + "行任务单号、区属、河湖名称关系错误");
            }
            // 判断水质数据表该任务、区属、河湖对应数据是否存在
            QueryWrapper dataWrapper = new QueryWrapper();
            dataWrapper.eq("mark", 1);
            dataWrapper.eq("inspect_driver_id", tauvInspectDriver.getId());
            TauvWaterData waterData = getOne(dataWrapper);
            if (waterData == null) {
                waterData = new TauvWaterData();
                waterData.setCreateUser(ShiroUtils.getAdminId());
                waterData.setCreateTime(DateUtil.now());
            } else {
                waterData.setUpdateUser(ShiroUtils.getAdminId());
                waterData.setUpdateTime(DateUtil.now());
            }
            waterData.setInspectDriverId(tauvInspectDriver.getId());
            waterData.setWaterCod(data.get(3).toString().trim());
            waterData.setWaterNh3n(data.get(4).toString().trim());
            waterData.setWaterTp(data.get(5).toString().trim());
            waterData.setWaterTn(data.get(6).toString().trim());
            waterData.setWaterDo(data.get(7).toString().trim());
            waterData.setWaterTub(data.get(8).toString().trim());
            saveOrUpdate(waterData);
            // 如果河湖选择了水质标准类型，则进行报警数据处理
            if (!StringUtils.isEmpty(driver.getCategory())) {
                dealWaterAlarm(waterData, driver);
            }
            m++;
        }
        return response.success("操作成功");
    }

    /**
     * 处理水质报警数据
     *
     * @param waterData
     * @return
     */
    private Boolean dealWaterAlarm(TauvWaterData waterData, TauvDriver driver) {
        QueryWrapper alarmWrapper = new QueryWrapper();
        alarmWrapper.eq("mark", 1);
        alarmWrapper.eq("inspect_driver_id", waterData.getInspectDriverId());
        alarmWrapper.last("limit 1");
        TauvWaterAlarm waterAlarm = waterAlarmMapper.selectOne(alarmWrapper);
        // 删除报警附表数据
        if (waterAlarm != null) {
            UpdateWrapper updateWrapper = new UpdateWrapper();
            updateWrapper.eq("water_alarm_id", waterAlarm.getId());
            updateWrapper.set("mark", 0);
            waterAlarmInfoService.update(updateWrapper);
            waterAlarm.setUpdateTime(DateUtil.now());
            waterAlarm.setUpdateUser(ShiroUtils.getAdminId());
        } else {
            waterAlarm = new TauvWaterAlarm();
            waterAlarm.setInspectDriverId(waterData.getInspectDriverId());
            waterAlarm.setCreateTime(DateUtil.now());
            waterAlarm.setCreateUser(ShiroUtils.getAdminId());
        }

        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        List<TauvWaterStandard> standardList = waterStandardMapper.selectList(wrapper);
        if (standardList.size() > 0) {
            List<String> alarmInfo = new ArrayList<>();
            List<TauvWaterAlarmInfo> waterAlarmInfoList = new ArrayList<>();
            Boolean flag = false;
            for (TauvWaterStandard item : standardList) {
                // 实测值
                BigDecimal value = BigDecimal.ZERO;
                switch (item.getCode().toLowerCase()) {
                    case "cod":
                        value = new BigDecimal(waterData.getWaterCod());
                        break;
                    case "nh3-n":
                        value = new BigDecimal(waterData.getWaterNh3n());
                        break;
                    case "tp":
                        value = new BigDecimal(waterData.getWaterTp());
                        break;
                    case "tn":
                        value = new BigDecimal(waterData.getWaterTn());
                        break;
                    case "do":
                        value = new BigDecimal(waterData.getWaterDo());
                        break;
                    case "tub":
                        value = new BigDecimal(waterData.getWaterTub());
                        break;
                    default:
                        break;
                }
                // 标准值
                BigDecimal standardValue = BigDecimal.ZERO;
                switch (driver.getCategory()) {
                    case "categoryOne":
                        standardValue = item.getCategoryOne();
                        break;
                    case "categoryTwo":
                        standardValue = item.getCategoryTwo();
                        break;
                    case "categoryThree":
                        standardValue = item.getCategoryThree();
                        break;
                    case "categoryFour":
                        standardValue = item.getCategoryFour();
                        break;
                    case "categoryFive":
                        standardValue = item.getCategoryFive();
                        break;
                    default:
                        break;
                }
                // 差值
                BigDecimal exceed = value.subtract(standardValue);
                TauvWaterAlarmInfo waterAlarmInfo = new TauvWaterAlarmInfo();
                waterAlarmInfo.setName(item.getName());
                waterAlarmInfo.setStandardValue(standardValue);
                waterAlarmInfo.setMeasuredValue(value);
                if (exceed.compareTo(BigDecimal.ZERO) > 0) {
                    flag = true;
                    BigDecimal rate = exceed.multiply(new BigDecimal("100")).divide(standardValue, 2, BigDecimal.ROUND_HALF_UP);
                    String exceedStandard = item.getName() + "超标" + rate + "%";
                    alarmInfo.add(exceedStandard);
                    // 超标详情
                    waterAlarmInfo.setMeasuredStandard(checkCategory(item, value));
                    waterAlarmInfo.setExceedStandard(exceedStandard);
                }
                waterAlarmInfoList.add(waterAlarmInfo);
            }
            // 报警数据入库
            if (flag) {
                TauvInspectDriver inspectDriver = inspectDriverMapper.selectById(waterData.getInspectDriverId());
                waterAlarm.setAlarmSn(createAlarmSn());
                waterAlarm.setDriverArea(inspectDriver.getDriverArea());
                waterAlarm.setDriverId(inspectDriver.getDriverId());
                waterAlarm.setDriverName(inspectDriver.getDriverName());
                waterAlarm.setInspectTime(inspectDriver.getExecutionTime());
                waterAlarm.setAlarmStandard(driver.getCategory());
                waterAlarm.setAlarmInfo(Joiner.on(",").join(alarmInfo));
                waterAlarmService.saveOrUpdate(waterAlarm);
                // 报警详情附表数据入库
                for (TauvWaterAlarmInfo data : waterAlarmInfoList) {
                    data.setWaterAlarmId(waterAlarm.getId());
                    data.setCreateUser(ShiroUtils.getAdminId());
                    data.setCreateTime(DateUtil.now());
                    waterAlarmInfoService.save(data);
                }
            } else if (StringUtils.isNotNull(waterAlarm.getId())) {
                // 新导入数据没有报警数据，删除原有报警数据
                waterAlarmService.deleteById(waterAlarm.getId(), waterAlarm);
            }
        }
        return true;
    }

    /**
     * 根据数值判断属于什么质量标准
     *
     * @param waterStandard
     * @param value
     * @return
     */
    private String checkCategory(TauvWaterStandard waterStandard, BigDecimal value) {
        String category = "";
        Map<String, BigDecimal> map = new HashMap<>();
        map.put("Ⅰ类", waterStandard.getCategoryOne());
        map.put("Ⅱ类", waterStandard.getCategoryTwo());
        map.put("Ⅲ类", waterStandard.getCategoryThree());
        map.put("Ⅳ类", waterStandard.getCategoryFour());
        map.put("Ⅴ类", waterStandard.getCategoryFive());
        if ("≤".equals(waterStandard.getMethod())) {
            map = FunctionUtils.mapSorter(map, true);
            for (Map.Entry<String, BigDecimal> entry : map.entrySet()) {
                if (value.compareTo(entry.getValue()) <= 0) {
                    category = entry.getKey();
                    break;
                }
            }
        } else {
            map = FunctionUtils.mapSorter(map, false);
            for (Map.Entry<String, BigDecimal> entry : map.entrySet()) {
                if (value.compareTo(entry.getValue()) >= 0) {
                    category = entry.getKey();
                    break;
                }
            }
        }
        if (category.equals("")) {
            category = "Ⅴ级";
        }
        return category;
    }

    /**
     * 生成报警编码
     *
     * @return
     */
    private String createAlarmSn() {
        Calendar date = Calendar.getInstance();
        String year = String.valueOf(date.get(Calendar.YEAR)).substring(2);
        String month = String.format("%02d", date.get(Calendar.MONTH) + 1);
        String code = "20" + year + month + (int) ((Math.random() * 9 + 1) * 100000);
        return code;
    }

    @Override
    public Response getTemplate() {
        String filename = "导入水质数据模板.xlsx";
        File filedir = new File(uploadFolder);
        if (!filedir.isDirectory()) {
            filedir.mkdirs();
        }
        String path = uploadFolder + "/" + filename;
        return response.success(path.replace(uploadFolder, uploadUrl + "/upload/"));
    }
}
