package com.taauav.common.util;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.math.BigDecimal;

import cn.hutool.core.date.DateUtil;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.mapper.SysAdminMapper;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.math.RoundingMode;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.MessageDigest;
import java.text.ParseException;
import java.util.*;
import java.text.SimpleDateFormat;

/**
 * 常用方法
 *
 * @author dyg
 */
public class FunctionUtils {
    /**
     * 获取系统当前时间
     *
     * @return
     */
    public static Integer getCurrentTime() {
        Long t = System.currentTimeMillis();
        Integer time = Integer.valueOf(String.valueOf(t).substring(0, 10));
        return time;
    }

    /**
     * MD5方法
     *
     * @param source
     * @return
     */
    public static String md5(byte[] source) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(source);
            StringBuffer buf = new StringBuffer();
            for (byte b : md.digest()) {
                buf.append(String.format("%02x", b & 0xff));
            }
            return buf.toString();
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 判断字符串是否为数字
     *
     * @param str
     * @return
     */
    public static Boolean isNumeric(String str) {
        String bigStr;
        try {
            bigStr = new BigDecimal(str).toString();
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    /**
     * 是否是Get请求
     *
     * @return
     */
    public static boolean isGet() {
        String method = getRequestMethod();
        String str = "get";
        if (str.equalsIgnoreCase(method)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 是否是post请求
     *
     * @return
     */
    public static boolean isPost() {
        String method = getRequestMethod();
        String str = "post";
        if (str.equalsIgnoreCase(method)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 获取请求方式
     *
     * @return
     */
    public static String getRequestMethod() {
        HttpServletRequest request = getRequest();
        return request.getMethod().toLowerCase();
    }

    /**
     * 获取http请求
     *
     * @return
     */
    public static HttpServletRequest getRequest() {
        return ((ServletRequestAttributes) (RequestContextHolder.getRequestAttributes())).getRequest();
    }

    public static HttpServletResponse getResponse() {
        return ((ServletRequestAttributes) (RequestContextHolder.getRequestAttributes())).getResponse();
    }

    /**
     * 密码加密
     *
     * @param password
     * @return
     */
    public static String password(String password) {
        String md51 = md5(password.getBytes());
        String pwd = md5((md51 + "IgtUdEQJyVevaCxQnY").getBytes());
        return pwd;
    }

    /**
     * 当前时间戳
     *
     * @return
     */
    public static Integer now() {
        long currentTime = System.currentTimeMillis();
        String t = String.valueOf(currentTime);
        String seconds = t.substring(0, 10);
        return Integer.valueOf(seconds);
    }

    /**
     * 时间戳转化为日期
     *
     * @param time
     * @param format
     * @return
     */
    public static String formatTime(Integer time, String format) {
        if (StringUtils.isEmpty(time) || time <= 0) {
            return "";
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat(format);
        long timeLong = time.longValue() * 1000;
        Date date = new Date(timeLong);
        return dateFormat.format(date);
    }

    /**
     * 日期转化为时间戳
     *
     * @param dateStr
     * @return
     * @throws ParseException
     */
    public static Integer dateToStamp(String dateStr) {
        Date date = DateUtil.parse(dateStr);
        long dateRes = date.getTime() / 1000;
        Integer stamp = new Long(dateRes).intValue();
        return stamp;
    }

    /**
     * 判断key是否在数组中
     *
     * @param key
     * @param map
     * @return
     */
    public static boolean inArray(String key, Map<String, String> map) {
        boolean flag = false;
        for (String k : map.keySet()) {
            if (k.equals(key)) {
                flag = true;
            }
        }
        return flag;
    }

    /**
     * 根据键获取数组中对应值
     *
     * @param key
     * @param map
     * @return
     */
    public static String getArrayText(String key, Map<String, String> map) {
        String text = "";
        String[] strArray = null;
        String str = ",";
        if (key.contains(str)) {
            strArray = key.split(str);
        } else {
            strArray = new String[]{key};
        }
        for (String k : strArray) {
            if (inArray(k, map)) {
                text += map.get(k) + ",";
            }
        }
        if (text.length() > 0) {
            text = text.substring(0, text.length() - 1);
        }
        return text;
    }

    /**
     * 对象转换成map
     *
     * @param obj
     * @return
     */
    public static Map<String, Object> transBean2Map(Object obj) {
        if (obj == null) {
            return null;
        }
        Map<String, Object> map = new HashMap<String, Object>();
        try {
            BeanInfo beanInfo = Introspector.getBeanInfo(obj.getClass());
            PropertyDescriptor[] propertyDescriptors = beanInfo.getPropertyDescriptors();
            for (PropertyDescriptor property : propertyDescriptors) {
                String key = property.getName();
                // 过滤class属性
                if (!"class".equals(key)) {
                    // 得到property对应的getter方法
                    Method getter = property.getReadMethod();
                    Object value = getter.invoke(obj);
                    map.put(key, value);
                }

            }
        } catch (Exception e) {
            System.out.println("transBean2Map Error " + e);
            e.printStackTrace();
        }
        return map;
    }

    /**
     * 根据指定的类型返回当前日期时间格式
     *
     * @param type
     * @return
     */
    public static String getTimeNameByType(String type) {
        type = StringUtils.isEmpty(type) ? "" : type;
        Calendar calendar = Calendar.getInstance();
        String year = calendar.get(Calendar.YEAR) + type;
        String month = (calendar.get(Calendar.MONTH) + 1) + type;
        String day = calendar.get(Calendar.DATE) + type;
        String hour = calendar.get(Calendar.HOUR) + type;
        String minute = calendar.get(Calendar.MINUTE) + type;
        String second = calendar.get(Calendar.SECOND) + type;
        String fileName = year + month + day + hour + minute + second;
        return fileName;
    }

    /**
     * 进一算法
     *
     * @param a
     * @param b
     * @return
     */
    public static Integer ceil(Integer a, Integer b) {
        return a % b > 0 ? a / b : (a / b + 1);
    }

    /**
     * 判断数组对应键的值是否存在
     *
     * @param key
     * @param map
     * @return Boolean true:不存在 false:存在
     */
    public static Boolean isEmpty(String key, Map<String, String> map) {
        if (map == null) {
            return true;
        } else if (!map.containsKey(key) || "".equals(map.get(key))) {
            return true;
        }
        return false;
    }

    /**
     * 判断是否为手机号
     *
     * @param mobile
     * @return
     */
    public static Boolean isMobile(String mobile) {
        String regex = "^1[3-9][0-9]{9}";
        return mobile.matches(regex);
    }

    /**
     * GET请求
     *
     * @param geturl
     * @param param
     * @return
     * @throws Exception
     */
    public static String getUrl(String geturl, String param) throws Exception {
        //请求的webservice的url
        URL url = new URL(geturl);
        //创建http链接
        HttpURLConnection httpURLConnection = (HttpURLConnection) url.openConnection();

        //设置请求的方法类型
        httpURLConnection.setRequestMethod("GET");

        //设置请求的内容类型
        httpURLConnection.setRequestProperty("Content-type", "application/x-www-form-urlencoded");

        //设置发送数据
        httpURLConnection.setDoOutput(true);
        //设置接受数据
        httpURLConnection.setDoInput(true);

        //发送数据,使用输出流
        OutputStream outputStream = httpURLConnection.getOutputStream();
        //发送的soap协议的数据
        //    String requestXmlString = requestXml("北京");

        String content = param;

        //发送数据
        outputStream.write(content.getBytes());

        //接收数据
        InputStream inputStream = httpURLConnection.getInputStream();
        BufferedReader in = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
        StringBuffer buffer = new StringBuffer();
        String line = "";
        while ((line = in.readLine()) != null) {
            buffer.append(line);
        }
        String str = buffer.toString();

        return str;
    }

    /**
     * 格式小数位数
     *
     * @param numberStr
     * @param scale
     * @return
     */
    public static String round(double numberStr, int scale) {
        BigDecimal bigDecimal = new BigDecimal(numberStr);
        double dt = bigDecimal.setScale(scale, RoundingMode.HALF_UP).doubleValue();
        return dt + "";
    }

    /**
     * 经纬度转换 ，度分秒转度
     *
     * @param jwd
     * @return
     * @author Cai_YF
     */
    public static String Dms2D(String jwd) {
        if (!StringUtils.isEmpty(jwd) && (jwd.contains("°"))) {//如果不为空并且存在度单位
            //计算前进行数据处理
            jwd = jwd.replace("E", "").replace("N", "").replace(":", "").replace("：", "");
            double d = 0, m = 0, s = 0;
            d = Double.parseDouble(jwd.split("°")[0]);
            //不同单位的分，可扩展
            if (jwd.contains("′")) {//正常的′
                m = Double.parseDouble(jwd.split("°")[1].split("′")[0]);
            } else if (jwd.contains("'")) {//特殊的'
                m = Double.parseDouble(jwd.split("°")[1].split("'")[0]);
            }
            //不同单位的秒，可扩展
            if (jwd.contains("″")) {//正常的″
                //有时候没有分 如：112°10.25″
                s = jwd.contains("′") ? Double.parseDouble(jwd.split("′")[1].split("″")[0]) : Double.parseDouble(jwd.split("°")[1].split("″")[0]);
            } else if (jwd.contains("''")) {//特殊的''
                //有时候没有分 如：112°10.25''
                s = jwd.contains("'") ? Double.parseDouble(jwd.split("'")[1].split("''")[0]) : Double.parseDouble(jwd.split("°")[1].split("''")[0]);
            } else if (jwd.contains("\"")) {
                s = jwd.contains("'") ? Double.parseDouble(jwd.split("'")[1].split("\"")[0]) : Double.parseDouble(jwd.split("°")[1].split("\"")[0]);
            }
            jwd = round(d + m / 60 + s / 60 / 60, 6);//计算并转换为string
        }
        return jwd;
    }

    private static SysAdminMapper adminMapper = SpringUtil.getBean(SysAdminMapper.class);

    /**
     * 处理区域条件
     *
     * @param map
     * @return
     */
    public static Map<String, Object> dealArea(Map<String, Object> map) {
        Integer adminId = ShiroUtils.getAdminId();
        SysAdmin admin = adminMapper.selectById(adminId);
        String[] areaIds;
        if (!StringUtils.isEmpty(map.get("driverArea"))) {
            areaIds = map.get("driverArea").toString().split(",");
        } else {
            areaIds = com.taauav.common.util.StringUtils.isEmpty(admin.getCityIds()) ? null : admin.getCityIds().split(",");
        }
        if (null != areaIds && areaIds.length > 0) {
            map.put("driverArea", areaIds);
        } else {
            map.remove("driverArea");
        }
        return map;
    }

    /**
     * object 转 List
     * @param object
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> List<T> castList(Object object, Class<T> clazz) {
        List<T> result = new ArrayList<>();
        if (object instanceof List<?>) {
            for (Object o : (List<?>) object) {
                result.add(clazz.cast(o));
            }
            return result;
        }
        return null;
    }

    /**
     * Map排序
     * @param map
     * @param flag true：正序（从小到大） false：反序（从大到小）
     * @return
     */
    public static Map<String, BigDecimal> mapSorter(Map<String, BigDecimal> map, Boolean flag) {
        List<Map.Entry<String, BigDecimal>> list = new ArrayList<>(map.entrySet());
        if (flag) {
            list.sort(Map.Entry.comparingByValue());
        } else {
            list.sort(Map.Entry.comparingByValue(Comparator.reverseOrder()));
        }
        Map<String, BigDecimal> newMap = new LinkedHashMap<String, BigDecimal>();
        for (int i = 0; i < list.size(); i++) {
            newMap.put(list.get(i).getKey(), list.get(i).getValue());
        }
        return newMap;
    }

    /**
     * 字符串转日期
     * @return
     */
    public static Date strToDate(String str, String formatStr) {
        if (StringUtils.isEmpty(formatStr)) {
            formatStr = "yyyy-MM-dd HH:mm:ss";
        }
        SimpleDateFormat format = new SimpleDateFormat(formatStr);
        Date date = null;
        try {
            date = format.parse(str);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return date;
    }
}
