package com.taauav.front.controller;


import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.front.entity.UserAuthGroup;
import com.taauav.front.query.UserAuthGroupQuery;
import com.taauav.front.service.IUserAuthGroupService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 外部用户角色表 前端控制器
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@RestController
@RequestMapping("/front/authgroup")
public class UserAuthGroupController extends FrontBaseController {

    @Autowired
    private IUserAuthGroupService userAuthGroupService;

    @Autowired
    private Response response;

    private static final String controllerName = "authgroup";

    /**
     * 获取角色列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody UserAuthGroupQuery query) {
        return userAuthGroupService.getList(query);
    }

    /**
     * 添加角色
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody(required = true) UserAuthGroup entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userAuthGroupService.add(entity);
    }

    /**
     * 获取角色详情
     *
     * @param id 角色ID
     * @return
     */
    @GetMapping("/info")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response info(Integer id) {
        return userAuthGroupService.info(id);
    }

    /**
     * 编辑角色
     *
     * @param entity        实体对象
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody(required = false) UserAuthGroup entity, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String message = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(message);
        }
        return userAuthGroupService.edit(entity);
    }

    /**
     * 删除角色
     *
     * @param id 角色ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return userAuthGroupService.drop(id);
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody(required = true) UserAuthGroup entity) {
        return userAuthGroupService.setStatus(entity);
    }

    /**
     * 获取角色列表
     *
     * @return
     */
    @PostMapping("/getAuthGroupList")
    public Response getAuthGroupList() {
        return userAuthGroupService.getAuthGroupList();
    }

}
