package com.taauav.front.service;

import com.taauav.common.bean.Response;
import com.taauav.common.service.IBaseService;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.query.UserAdminQuery;
import com.taauav.front.vo.UserAdminVo;
import com.taauav.front.vo.UserVillageVo;
import com.taauav.front.vo.SysCityVo;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 外包人员表 服务类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
public interface IUserAdminService extends IBaseService<UserAdmin> {

    /**
     * 获取用户列表
     *
     * @param query 查询条件
     * @return
     */
    Response getList(UserAdminQuery query);

    /**
     * 添加用户
     *
     * @param entity 实体对象
     * @return
     */
    Response add(UserAdmin entity);

    /**
     * 获取数据详情
     *
     * @param id 用户ID
     * @return
     */
    Response info(Integer id);

    /**
     * 编辑用户
     *
     * @param entity 实体对象
     * @return
     */
    Response edit(UserAdmin entity);

    /**
     * 删除用户
     *
     * @param id 用户ID
     * @return
     */
    Response drop(Integer id);

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    Response setStatus(UserAdmin entity);

    /**
     * 重置密码
     *
     * @param entity 实体对象
     * @return
     */
    Response resetPwd(UserAdmin entity);

    /**
     * 基础数据
     *
     * @return
     */
    Map<String, Object> getBaseData();

    /**
     * 获取用户登录权限
     *
     * @param userId 用户ID
     * @return
     */
    Map<Integer, String> getAdminAuthData(Integer userId);

    /**
     * 获取河长列表
     *
     * @return
     */
    Response getDriverManagerList();

    /**
     * 获取用户信息
     *
     * @param userId
     * @return
     */
    UserAdminVo getAdminInfo(Integer userId);

    /**
     * 获取用户有查看权限区域id
     *
     * @param adminInfo
     * @param map
     * @return
     */
    List<SysCityVo> getAdminAreaList(UserAdminVo adminInfo, Map<String, String> map);

    /**
     * 根据镇/街道id获取权限内下属村数据
     *
     * @param userId
     * @param cityId
     * @param map
     * @return
     */
    UserVillageVo getVillageList(Integer userId, BigInteger cityId, Map<String, String> map);

//    /**
//     * 获取区域内所有河道问题坐标点
//     *
//     * @param inspectFileDTO
//     * @return
//     */
//    Response getQuestionList(LsInspectFileDTO inspectFileDTO);

    /**
     * 根据用户获取数据
     *
     * @param username
     * @return
     */
    UserAdmin findByUsername(String username);

    /**
     * 获取人员列表
     *
     * @return
     */
    Response getUserAdminList();

    /**
     * 根据用户编号获取数据
     *
     * @param id
     * @return
     */
    UserAdmin findById(Integer id);

}
