package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FileUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.front.constant.UserInspectQuestionConstant;
import com.taauav.front.dto.inspectquestion.InspectQuestionDealwithDto;
import com.taauav.front.dto.inspectquestion.InspectQuestionDto;
import com.taauav.front.entity.UserInspectQuestion;
import com.taauav.front.mapper.UserInspectQuestionMapper;
import com.taauav.front.query.UserInspectQuestionQuery;
import com.taauav.front.service.IUserInspectQuestionService;
import com.taauav.front.vo.inspectquestion.UserInspectQuestionInfoVo;
import com.taauav.front.vo.inspectquestion.UserInspectQuestionListVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.List;

/**
 * <p>
 * 问题管理 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@Service
public class UserInspectQuestionServiceImpl extends BaseServiceImpl<UserInspectQuestionMapper, UserInspectQuestion> implements IUserInspectQuestionService {

    @Autowired
    private UserInspectQuestionMapper inspectQuestionMapper;

    @Autowired
    private Response response;

    @Autowired
    private ISysCityService sysCityService;

    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Value("${file.uploadFolder}")
    private String uploadFolder;

    /**
     * 获取巡检问题列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserInspectQuestionQuery query) {
        // 区划ID处理
        if (query.getDriverAreaId() != null) {
            List<BigInteger> cityIds = sysCityService.getChildCityIds(query.getDriverAreaId());
            // 加入当前选中的区划
            cityIds.add(query.getDriverAreaId());
            BigInteger[] integers = cityIds.toArray(new BigInteger[cityIds.size()]);
            query.setDriverArea(integers);
        }
        // 巡检时间处理
        if (!StringUtils.isEmpty(query.getInspectTime())) {
            String startTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 0:00:00";
            String endTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 23:59:59";
            query.setInspectStartTime(startTime);
            query.setInspectEndTime(endTime);
        }
        IPage<UserInspectQuestionListVo> page = new Page<>(query.getPage(), query.getPage());
        List<UserInspectQuestionListVo> inspectQuestionList = inspectQuestionMapper.getInspectQuestionList(page, query);
        if (!inspectQuestionList.isEmpty()) {
            inspectQuestionList.forEach(item -> {
                // 状态描述
                item.setStatusName(UserInspectQuestionConstant.INSPECT_QUESTION_STATUS_LIST.get(item.getStatus()));
                // 图片处理
                if (!StringUtils.isEmpty(item.getFileSrc())) {
                    item.setFileSrc(uploadUrl + item.getFileSrc());
                }
            });
        }
        page.setRecords(inspectQuestionList);
        return response.success(page);
    }

    /**
     * 问题指派
     *
     * @param inspectQuestionDto 参数
     * @return
     */
    @Override
    public Response assign(InspectQuestionDto inspectQuestionDto) {
        if (inspectQuestionDto.getId() == null || inspectQuestionDto.getId() <= 0) {
            return response.failure("巡检问题ID不能为空");
        }
        if (StringUtils.isEmpty(inspectQuestionDto.getAssignUser())) {
            return response.failure("责任人不能为空");
        }
        UserInspectQuestion inspectQuestion = inspectQuestionMapper.selectById(inspectQuestionDto.getId());
        if (inspectQuestion == null) {
            return response.failure("巡检问题信息不存在");
        }
        if (inspectQuestion.getStatus() == 3) {
            return response.failure("巡检问题已处理完成,无法指派");
        }
        UserInspectQuestion entity = new UserInspectQuestion();
        entity.setId(inspectQuestionDto.getId());
        entity.setAssignUser(inspectQuestionDto.getAssignUser());
        entity.setAssignContact(inspectQuestionDto.getAssignContact());
        entity.setAssignNote(inspectQuestionDto.getAssignNote());
        entity.setAssignTime(DateUtil.now());
        entity.setStatus(2);
        boolean result = editData(entity);
        if (!result) {
            return response.failure("问题指派失败");
        }
        return response.success("问题指派成功");
    }

    /**
     * 问题处理
     *
     * @param inspectQuestionDealwithDto 参数
     * @return
     */
    @Override
    public Response dealWith(InspectQuestionDealwithDto inspectQuestionDealwithDto) {
        if (inspectQuestionDealwithDto.getId() == null || inspectQuestionDealwithDto.getId() <= 0) {
            return response.failure("巡检问题ID不能为空");
        }
        if (StringUtils.isEmpty(inspectQuestionDealwithDto.getHandlerImage())) {
            return response.failure("处理结果图片不能为空");
        }
        UserInspectQuestion inspectQuestion = inspectQuestionMapper.selectById(inspectQuestionDealwithDto.getId());
        if (inspectQuestion == null) {
            return response.failure("巡检问题信息不存在");
        }
        if (inspectQuestion.getStatus() == 1) {
            return response.failure("巡检问题未指派,无法处理");
        }
        if (inspectQuestion.getStatus() == 3) {
            return response.failure("巡检问题已处理完成,无需再次处理");
        }
        UserInspectQuestion entity = new UserInspectQuestion();
        entity.setId(inspectQuestionDealwithDto.getId());
        entity.setHandlerUser(ShiroUtils.getAdminId());
        entity.setHandlerResult(inspectQuestionDealwithDto.getHandlerResult());
        entity.setHandlerTime(DateUtil.now());
        entity.setStatus(3);

        // 如果上传了文件则对文件进行处理
        String file = inspectQuestionDealwithDto.getHandlerImage();
        if (file.contains("temp")) {
            file = file.replace(uploadUrl, uploadFolder);
            File oldFile = new File(file);
            if (!oldFile.exists()) {
                return response.failure("文件不存在");
            }
            //把文件移动到正式目录下
            Calendar date = Calendar.getInstance();
            String year = String.valueOf(date.get(Calendar.YEAR));
            String month = String.valueOf(date.get(Calendar.MONTH) + 1);
            String day = String.valueOf(date.get(Calendar.DATE));
            String destFile = uploadFolder + "file/file/" + year + "/" + month + "/" + day + "/";
            boolean flag = FileUtil.cutGeneralFile(file, destFile);
            if (flag) {
                // 文件名
                String fileName = oldFile.getName();
                destFile = destFile.replace(uploadFolder, "");
                entity.setHandlerImage(destFile + fileName);
            } else {
                return response.failure("文件保存失败了");
            }
        }
        boolean result = editData(entity);
        if (!result) {
            return response.failure("问题处理失败");
        }
        return response.success("问题处理成功");
    }

    /**
     * 获取问题详情
     *
     * @param id 问题ID
     * @return
     */
    @Override
    public Response detail(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("问题ID不能为空");
        }
        UserInspectQuestion entity = inspectQuestionMapper.selectById(id);
        if (entity == null) {
            return response.failure("巡检问题信息不存在");
        }
        UserInspectQuestionInfoVo inspectQuestionInfoVo = inspectQuestionMapper.getInspectQuestionInfoById(id);
        return response.success(inspectQuestionInfoVo);
    }

    /**
     * 标记完成
     *
     * @param id 问题ID
     * @return
     */
    @Override
    public Response funished(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("问题ID不能为空");
        }
        UserInspectQuestion entity = inspectQuestionMapper.selectById(id);
        if (entity == null) {
            return response.failure("巡检问题信息不存在");
        }
        entity.setStatus(3);
        boolean result = this.editData(entity);
        if (!result) {
            return response.failure("操作失败");
        }
        return response.success("操作成功");
    }
}
