package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvInspect;
import com.taauav.admin.mapper.TauvInspectMapper;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FileUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.front.constant.UserNoticeConstant;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.entity.UserNotice;
import com.taauav.front.mapper.UserAdminMapper;
import com.taauav.front.mapper.UserNoticeMapper;
import com.taauav.front.query.UserNoticeQuery;
import com.taauav.front.service.IUserInspectDriverService;
import com.taauav.front.service.IUserInspectService;
import com.taauav.front.service.IUserNoticeService;
import com.taauav.front.vo.usernotice.UserNoticeInfoVo;
import com.taauav.front.vo.usernotice.UserNoticeListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.util.*;

/**
 * <p>
 * 通知公告表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@Service
public class UserInspectServiceImpl extends BaseServiceImpl<TauvInspectMapper, TauvInspect> implements IUserInspectService {

    @Autowired
    private Response response;

    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Value("${file.uploadFolder}")
    private String uploadFolder;

    @Autowired
    private IUserInspectDriverService userInspectDriverService;

    /**
     * 创建巡检任务
     *
     * @param inspect
     * @return
     */
    @Override
    public Response addTask(TauvInspect inspect) {
        // 巡检类型校验
        if (inspect.getType() == null || inspect.getType() <= 0) {
            return response.failure("请选择巡检类型");
        }
        // 创建巡检任务
        boolean result = this.addData(inspect);
        if (!result) {
            return response.failure("巡检任务创建失败");
        }

        // 生成巡检任务单号
        String code = "";
        if (inspect.getType() == 1) {
            // 可见光巡检
            code = "KJG" + FunctionUtils.formatTime(FunctionUtils.getCurrentTime(), "yyyyMM");
        } else if (inspect.getType() == 2) {
            // 多光谱巡检
            code = "DGP" + FunctionUtils.formatTime(FunctionUtils.getCurrentTime(), "yyyyMM");
        }
        Integer id = inspect.getId();
        String idStr = String.format("%04d", id);
        inspect.setId(id);
        inspect.setInspectNo(code + idStr);

        // 图片处理
        if (!StringUtils.isEmpty(inspect.getAttachment())) {
            // 如果上传了文件则对文件进行处理
            String file = inspect.getAttachment();
            if (file.contains("temp")) {
                file = file.replace(uploadUrl, uploadFolder);
                File oldFile = new File(file);
                if (!oldFile.exists()) {
                    return response.failure("文件不存在");
                }
                //把文件移动到正式目录下
                Calendar date = Calendar.getInstance();
                String year = String.valueOf(date.get(Calendar.YEAR));
                String month = String.valueOf(date.get(Calendar.MONTH) + 1);
                String day = String.valueOf(date.get(Calendar.DATE));
                String destFile = uploadFolder + "file/file/" + year + "/" + month + "/" + day + "/";
                boolean flag = FileUtil.cutGeneralFile(file, destFile);
                if (flag) {
                    // 文件名
                    String fileName = oldFile.getName();
                    destFile = destFile.replace(uploadFolder, "");
                    inspect.setAttachment(destFile + fileName);
                } else {
                    return response.failure("文件保存失败了");
                }
            }
        }
        saveOrUpdate(inspect);
        String msg = userInspectDriverService.batchAddData(inspect);
        if (!"".equals(msg)) {
            return response.failure(msg);
        }
        return response.success("添加成功");
    }
}
