package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.common.bean.CacheUser;
import com.taauav.common.bean.Response;
import com.taauav.common.config.FrontUserToken;
import com.taauav.front.dto.LoginDto;
import com.taauav.front.entity.UserAdmin;
import com.taauav.front.mapper.UserAdminMapper;
import com.taauav.front.service.IUserLoginService;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 外部人员表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2020-04-01
 */
@Service
public class UserLoginServiceImpl extends ServiceImpl<UserAdminMapper, UserAdmin> implements IUserLoginService {

    @Autowired
    private Response response;

    @Autowired
    private UserAdminMapper userAdminMapper;

    /**
     * 系统登录
     *
     * @param loginDto 参数
     * @return
     */
    @Override
    public Response login(LoginDto loginDto) {
        // 获取Subject实例对象，用户实例
        Subject currentUser = SecurityUtils.getSubject();

        // 将用户名和密码封装到UsernamePasswordToken
        UsernamePasswordToken token = new FrontUserToken(loginDto.getUsername(), loginDto.getPassword());
        CacheUser cacheUser;
        // 4、认证
        try {
            // 传到 MyShiroRealm 类中的方法进行认证
            currentUser.login(token);
            // 构建缓存用户信息返回给前端
            UserAdmin user = (UserAdmin) currentUser.getPrincipals().getPrimaryPrincipal();
            cacheUser = CacheUser.builder()
                    .token(currentUser.getSession().getId().toString())
                    .build();
            BeanUtils.copyProperties(user, cacheUser);
        } catch (UnknownAccountException e) {
            log.error("账户不存在异常：", e);
            return response.failure("账号不存在!");
//            throw new LoginException("账号不存在!", e);
        } catch (IncorrectCredentialsException e) {
            log.error("凭据错误（密码错误）异常：", e);
            return response.failure("密码不正确!");
//            throw new LoginException("密码不正确!", e);
        } catch (AuthenticationException e) {
            log.error("身份验证异常:", e);
            return response.failure("用户验证失败!");
//            throw new LoginException("用户验证失败!", e);
        }
        Map<String, String> result = new HashMap<>();
        result.put("token", cacheUser.getToken());
        return response.success("登录成功", result);
    }

}
