package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterData;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.front.mapper.UserWaterDataMapper;
import com.taauav.front.query.UserWaterDataQuery;
import com.taauav.front.service.IUserWaterDataService;
import com.taauav.front.vo.waterdata.UserWaterDataListVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigInteger;
import java.util.List;

/**
 * <p>
 * 水质数据表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-15
 */
@Service
public class UserWaterDataServiceImpl extends BaseServiceImpl<UserWaterDataMapper, TauvWaterData> implements IUserWaterDataService {

    @Autowired
    private ISysCityService sysCityService;

    @Autowired
    private UserWaterDataMapper userWaterDataMapper;

    @Autowired
    private Response response;

    /**
     * 获取水质数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserWaterDataQuery query) {
        // 区划ID处理
        if (query.getDriverArea() != null) {
            List<BigInteger> cityIds = sysCityService.getChildCityIds(query.getDriverArea());
            // 加入当前选中的区划
            cityIds.add(query.getDriverArea());
            BigInteger[] integers = cityIds.toArray(new BigInteger[cityIds.size()]);
            query.setDriverAreaList(integers);
        }
        // 巡检时间处理
        if (!StringUtils.isEmpty(query.getInspectTime())) {
            String startTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 0:00:00";
            String endTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 23:59:59";
            query.setInspectStartTime(startTime);
            query.setInspectEndTime(endTime);
        }
        IPage<UserWaterDataListVo> page = new Page<>(query.getPage(), query.getPage());
        List<UserWaterDataListVo> inspectQuestionList = userWaterDataMapper.getWaterDataList(page, query);
        page.setRecords(inspectQuestionList);
        return response.success(page);
    }
}
