package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterSpectrum;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.front.mapper.UserWaterSpectrumMapper;
import com.taauav.front.query.UserWaterSpectrumQuery;
import com.taauav.front.service.IUserWaterSpectrumService;
import com.taauav.front.vo.userwaterspectrum.UserWaterSpectrumListVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigInteger;
import java.util.List;

/**
 * <p>
 * 多光谱图表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-15
 */
@Service
public class UserWaterSpectrumServiceImpl extends BaseServiceImpl<UserWaterSpectrumMapper, TauvWaterSpectrum> implements IUserWaterSpectrumService {

    @Autowired
    private UserWaterSpectrumMapper userWaterSpectrumMapper;

    @Autowired
    private ISysCityService sysCityService;

    @Autowired
    private Response response;

    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;

    /**
     * 获取多光谱数据列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserWaterSpectrumQuery query) {
        // 区划ID处理
        if (query.getDriverArea() != null) {
            List<BigInteger> cityIds = sysCityService.getChildCityIds(query.getDriverArea());
            // 加入当前选中的区划
            cityIds.add(query.getDriverArea());
            BigInteger[] integers = cityIds.toArray(new BigInteger[cityIds.size()]);
            query.setDriverAreaList(integers);
        }
        // 巡检时间处理
        if (!StringUtils.isEmpty(query.getInspectTime())) {
            String startTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 0:00:00";
            String endTime = FunctionUtils.formatTime(Integer.valueOf(query.getInspectTime()), "yyyy-MM-dd") + " 23:59:59";
            query.setInspectStartTime(startTime);
            query.setInspectEndTime(endTime);
        }
        IPage<UserWaterSpectrumListVo> page = new Page<>(query.getPage(), query.getPage());
        List<UserWaterSpectrumListVo> waterSpectrumListVoList = userWaterSpectrumMapper.getWaterSpectrumList(page, query);
        if (!waterSpectrumListVoList.isEmpty()) {
            waterSpectrumListVoList.forEach(item -> {
                // 化学需氧量多光谱路径
                if (!StringUtils.isEmpty(item.getCodPic())) {
                    item.setCodPic(uploadUrl + item.getCodPic());
                }
                // 氨氮多光谱路径
                if (!StringUtils.isEmpty(item.getNh3nPic())) {
                    item.setNh3nPic(uploadUrl + item.getNh3nPic());
                }
                // 总磷多光谱路径
                if (!StringUtils.isEmpty(item.getTpPic())) {
                    item.setTpPic(uploadUrl + item.getTpPic());
                }
                // 总氮多光谱路径
                if (!StringUtils.isEmpty(item.getTnPic())) {
                    item.setTnPic(uploadUrl + item.getTnPic());
                }
                // 溶解氧多光谱路径
                if (!StringUtils.isEmpty(item.getDoPic())) {
                    item.setDoPic(uploadUrl + item.getDoPic());
                }
                // 浊度多光谱路径
                if (!StringUtils.isEmpty(item.getTubPic())) {
                    item.setTubPic(uploadUrl + item.getTubPic());
                }
            });
        }
        page.setRecords(waterSpectrumListVoList);
        return response.success(page);
    }
}
