package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterWarn;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.front.constant.UserWaterWarnConstant;
import com.taauav.front.mapper.UserWaterWarnMapper;
import com.taauav.front.query.UserWaterWarnQuery;
import com.taauav.front.service.IUserWaterWarnService;
import com.taauav.front.vo.waterwarn.UserWaterWarnListVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 水质预警 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-15
 */
@Service
public class UserWaterWarnServiceImpl extends BaseServiceImpl<UserWaterWarnMapper, TauvWaterWarn> implements IUserWaterWarnService {

    @Autowired
    private UserWaterWarnMapper userWaterWarnMapper;

    @Autowired
    private ISysCityService sysCityService;

    @Autowired
    private Response response;

    /**
     * 获取水质预警列表
     *
     * @param query 查询条件
     * @return
     */
    @Override
    public Response getList(UserWaterWarnQuery query) {
        // 查询条件
        QueryWrapper<TauvWaterWarn> queryWrapper = new QueryWrapper<>();
        // 预警编号
        if (!StringUtils.isEmpty(query.getWarnSn())) {
            queryWrapper.like("warn_sn", query.getWarnSn());
        }
        // 区划ID
        if (query.getDriverArea() != null) {
            // 获取所有自己区划ID
            List<BigInteger> driverAreaList = new ArrayList<>();
            driverAreaList.add(query.getDriverArea());
            List<BigInteger> cityIdsList = sysCityService.getChildCityIds(query.getDriverArea());
            if (!cityIdsList.isEmpty()) {
                cityIdsList.forEach(item -> {
                    driverAreaList.add(item);
                });
            }
            queryWrapper.in("driver_area", driverAreaList);
        }
        // 河湖名称
        if (!StringUtils.isEmpty(query.getDriverName())) {
            queryWrapper.like("driver_name", query.getDriverName());
        }
        // 状态
        if (query.getStatus() != null && query.getStatus() > 0) {
            queryWrapper.eq("status", query.getStatus());
        }
        // 预警时间
        if (!StringUtils.isEmpty(query.getWarnTime())) {
            String startTime = FunctionUtils.formatTime(Integer.valueOf(query.getWarnTime()), "yyyy-MM-dd") + " 0:00:00";
            String endTime = FunctionUtils.formatTime(Integer.valueOf(query.getWarnTime()), "yyyy-MM-dd") + " 23:59:59";
            queryWrapper.between("create_time", startTime, endTime);
        }
        IPage<TauvWaterWarn> page = new Page<>(query.getPage(), query.getPageSize());
        IPage<TauvWaterWarn> data = userWaterWarnMapper.selectPage(page, queryWrapper);
        List<TauvWaterWarn> waterAlarmList = data.getRecords();
        List<UserWaterWarnListVo> waterAlarmListVoList = new ArrayList<>();
        if (!waterAlarmList.isEmpty()) {
            waterAlarmList.forEach(item -> {
                UserWaterWarnListVo userWaterWarnListVo = new UserWaterWarnListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, userWaterWarnListVo);
                // 区划名称
                String cityName = sysCityService.getCityName(userWaterWarnListVo.getDriverArea());
                userWaterWarnListVo.setDriverAreaName(cityName);
                // 预警状态
                if (userWaterWarnListVo.getStatus() != null && userWaterWarnListVo.getStatus() > 0) {
                    userWaterWarnListVo.setStatusName(UserWaterWarnConstant.USER_WATER_WARN_STATUS_LIST.get(userWaterWarnListVo.getStatus()));
                }
                // 预警类型
                if (userWaterWarnListVo.getWarnType() != null && userWaterWarnListVo.getWarnType() > 0) {
                    userWaterWarnListVo.setWarnTypeName(UserWaterWarnConstant.USER_WATER_WARN_TYPE_LIST.get(userWaterWarnListVo.getWarnType()));
                }
                // 预警级别
                if (userWaterWarnListVo.getWarnLevel() != null && userWaterWarnListVo.getWarnLevel() > 0) {
                    userWaterWarnListVo.setWarnLevelName(UserWaterWarnConstant.USER_WATER_WARN_LEVEL_LIST.get(userWaterWarnListVo.getWarnLevel()));
                }
                waterAlarmListVoList.add(userWaterWarnListVo);
            });
        }
        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("total", data.getTotal());
        result.put("size", data.getSize());
        result.put("current", data.getCurrent());
        result.put("pages", data.getPages());
        result.put("records", waterAlarmListVoList);
        return response.success(result);
    }

    /**
     * 取消预警
     *
     * @param ids 记录ID
     * @return
     */
    @Override
    public Response cancel(String ids) {
        if (StringUtils.isEmpty(ids)) {
            return response.failure("预警记录ID不能为空");
        }
        if (ids.contains(",")) {
            Integer totalNum = 0;
            String[] strings = ids.split(",");
            for (String string : strings) {
                TauvWaterWarn entity = userWaterWarnMapper.selectById(ids);
                if (entity == null) {
                    continue;
                }
                entity.setStatus(3);
                boolean result = this.editData(entity);
                if (result) {
                    totalNum++;
                }
            }
            return response.success(String.format("本次共取消【%s】条预警数据", totalNum));
        } else {
            TauvWaterWarn entity = userWaterWarnMapper.selectById(ids);
            if (entity == null) {
                return response.failure("水质预警信息不存在");
            }
            entity.setStatus(3);
            boolean result = this.editData(entity);
            if (!result) {
                return response.failure("操作失败");
            }
            return response.success("本次共取消【1】条预警数据");
        }
    }
}
